import os
from json import JSONDecodeError
from typing import AnyStr, Dict, List, Optional

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class SDMSAPIError(OSDUAPIError):
    pass


class SDMSTokenAPI:
    def create_impersonation_token(
        self, *, user_token: AnyStr, resources: List[Dict], metadata: Dict
    ) -> Dict:
        headers = self.osdu_auth_backend.headers
        headers["user-token"] = user_token
        url = os.path.join(
            self.osdu_auth_backend.base_url, self.service_path, "impersonation-token"
        )
        request_body = {"resources": resources, "metadata": metadata}
        response = requests.post(
            url=url, headers=headers, json=request_body
        )
        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def refresh_impersonation_token(
        self, *, impersonation_token: AnyStr, impersonation_token_context: AnyStr
    ) -> Dict:
        headers = self.osdu_auth_backend.headers
        headers["impersonation-token"] = impersonation_token
        headers["impersonation-token-context"] = impersonation_token_context
        url = os.path.join(
            self.osdu_auth_backend.base_url, self.service_path, "impersonation-token"
        )
        response = requests.put(url=url, headers=headers)
        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()


class SDMSAppsAPI:
    def register_app(self, *, email: AnyStr, sdpath: AnyStr) -> Dict:
        params = {"email": email, "sdpath": sdpath}
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "app",
        )
        response = requests.post(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )
        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def retrive_registered_apps(self, *, sdpath: AnyStr) -> Dict:
        params = {"sdpath": sdpath}
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "app",
        )
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )
        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return {"apps": response.json()}

    def set_trusted_app(self, *, email: AnyStr, sdpath: AnyStr) -> Dict:
        params = {"email": email, "sdpath": sdpath}
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "app/trusted",
        )
        response = requests.post(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )
        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def retrive_trusted_apps(self, *, sdpath: AnyStr) -> Dict:
        params = {"sdpath": sdpath}
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "app/trusted",
        )
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )
        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return {"apps": response.json()}


class SDMSDatasetAPI:
    def retrieve_dataset(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        path: Optional[AnyStr] = "/",
        fetch_meta: bool = False,
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}",
        )
        # fill path & meta params
        params = {"path": path, "seismicmeta": fetch_meta}
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def retrieve_dataset_permission(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        path: Optional[AnyStr] = "/",
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}/permission",
        )
        # fill path & meta params
        params = {"path": path}
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def delete_dataset(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        path: Optional[AnyStr] = "/",
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}",
        )
        # fill path & meta params
        params = {"path": path}
        response = requests.delete(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def list_subprojects_datasets(
        self, *, tenant_id: AnyStr, subproject_id: AnyStr
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}",
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def register_dataset(
        self,
        *,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        legal_tags: List[AnyStr],
        gtags: List[AnyStr],
        other_relevant_data_countries: List[AnyStr],
        data: Dict,
        type: AnyStr,
        slm: Dict = {},
        path: AnyStr = None,
        kind: AnyStr = None,
        id: AnyStr = None,
        parents: List[AnyStr] = [],
    ) -> Dict:
        tenant_id, *_ = kind.split(":")

        request_body = {
            "type": type,
            "gtags": gtags,
            "seismicmeta": {
                "ancestry": {"parents": parents},
                "kind": kind,
                "legal": {
                    "legaltags": legal_tags,
                    "otherRelevantDataCountries": other_relevant_data_countries,
                },
                "slm": slm,
                "data": data,
            },
        }

        if id:
            _prefix = kind.rsplit(":", 1)[0].replace("wks:", "")
            request_body["seismicmeta"]["id"] = f"{_prefix}:{id}"

        params = {"path": path, "seismicmeta": True}
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}",
        )
        response = requests.post(
            url=url,
            headers=self.osdu_auth_backend.headers,
            json=request_body,
            params=params,
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def patch_datasets_metadata(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        metadata: Dict = {},
        filemetadata: Dict = {},
        seismicmeta: Dict = None,
        path: AnyStr = None,
        close: AnyStr = None,
    ) -> Dict:
        request_body = {
            "metadata": metadata,
            "filemetadata": filemetadata,
        }
        if seismicmeta:
            request_body["seismicmeta"] = seismicmeta
        params = {"path": path, "close": close}
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}",
        )
        response = requests.patch(
            url=url,
            headers=self.osdu_auth_backend.headers,
            json=request_body,
            params=params,
        )

        if not response.ok:
            raise SDMSAPIError(
                status_code=response.status_code, message=response.json()
            )

        return response.json()

    def lock_dataset(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        path: AnyStr = None,
        openmode: AnyStr = "write",
        wid: AnyStr = None,
    ) -> Dict:

        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}/lock",
        )
        params = {"path": path, "openmode": openmode, "wid": wid}
        response = requests.put(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def unlock_dataset(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        path: AnyStr,
        openmode: AnyStr,
    ) -> Dict:

        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"dataset/tenant/{tenant_id}/subproject/{subproject_id}/dataset/{dataset_id}/unlock",
        )
        params = {"path": path, "openmode": openmode}
        response = requests.put(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()


class SDMSUtilityAPI:
    def generate_gcs_access_token(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        dataset_id: AnyStr,
        dataset_path: AnyStr = "/",
        readonly=False,
    ) -> Dict:

        url = os.path.join(
            self.osdu_auth_backend.base_url,
            "api/seismic-store/v3/utility/gcs-access-token",
        )
        # NOTE: shortcircuit the sd path logic, to use generic subproject sd:// for token access
        # if dataset_path == "/":
        #    sdpath = f"sd://{tenant_id}/{subproject_id}/{dataset_id}"
        # else:
        #    # sdpath = f"sd://{tenant_id}/{subproject_id}/{dataset_path}/{dataset_id}"
        #    sdpath = os.path.join("sd://", tenant_id, subproject_id)
        #    print(sdpath)
        #    sdpath = os.path.join(sdpath, dataset_path.strip("/"))
        #    print(sdpath)
        #    sdpath = os.path.join(sdpath, dataset_id)

        sdpath = f"sd://{tenant_id}/{subproject_id}"
        params = {
            "sdpath": sdpath,
            "readonly": readonly,
        }
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            try:
                raise SDMSAPIError(
                    status_code=response.status_code, message=response.json()
                )
            except JSONDecodeError:
                raise SDMSAPIError(
                    status_code=response.status_code, message=response.text
                )

        return response.json()


class SDMSubprojectAPI:
    def create_new_subproject(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        admin: AnyStr,
        storage_class: AnyStr = None,
        storage_location: AnyStr = None,
        legal_tags: AnyStr = None,
        acls: Dict = None,
    ) -> Dict:
        request_body = {
            "admin": admin,
            "storage_class": storage_class or "REGIONAL",
            "storage_location": (storage_location).upper(),
        }
        if acls:
            request_body["acls"] = acls

        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"subproject/tenant/{tenant_id}/subproject/{subproject_id}",
        )

        headers = self.osdu_auth_backend.headers
        # put legal tags into headers
        if legal_tags:
            headers.update({"ltag": legal_tags})

        response = requests.post(
            url=url,
            headers=headers,
            json=request_body,
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def get_sdms_subprojects(self, *, tenant_id: AnyStr) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"subproject/tenant/{tenant_id}",
        )

        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return {"subprojects": response.json()}

    def get_sdms_subproject(
        self, *, tenant_id: AnyStr, subproject_id: AnyStr, translate_user_info=True
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"subproject/tenant/{tenant_id}",
            f"subproject/{subproject_id}",
        )
        query = {"translate-user-info": translate_user_info}

        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, query=query
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def delete_sdms_subproject(
        self, *, tenant_id: AnyStr, subproject_id: AnyStr
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"subproject/tenant/{tenant_id}",
            f"subproject/{subproject_id}",
        )
        response = requests.delete(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def patch_sdms_subproject_metadata(
        self,
        *,
        tenant_id: AnyStr,
        subproject_id: AnyStr,
        ltag: AnyStr = None,
        acls: Dict = None,
        access_policy: AnyStr = None,
        recursive: AnyStr = None,
    ) -> Dict:
        requests_body = {}
        if acls:
            requests_body["acls"] = acls
        if access_policy:
            requests_body["access_policy"] = access_policy

        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"subproject/tenant/{tenant_id}",
            f"subproject/{subproject_id}",
        )
        headers = self.osdu_auth_backend.headers
        if ltag:
            headers["ltag"] = ltag
        query = {}
        if recursive:
            query["recursive"] = recursive

        response = requests.patch(
            url=url, json=requests_body, headers=headers, params=query
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()


class SDMSTenantAPI:
    def register_sdms_tenant(
        self, *, tenant_id: AnyStr, gcpid: AnyStr, esd: AnyStr, default_acl: AnyStr
    ) -> Dict:
        requests_body = {"gcpid": gcpid, "esd": esd, "default_acl": default_acl}
        url = os.path.join(
            self.osdu_auth_backend.base_url, self.service_path, f"tenant/{tenant_id}"
        )
        response = requests.post(
            url=url, json=requests_body, headers=self.osdu_auth_backend.headers
        )

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()

    def get_sdms_tenant(self, *, tenant_id: AnyStr) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url, self.service_path, f"tenant/{tenant_id}"
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise SDMSAPIError(status_code=response.status_code, message=response.text)

        return response.json()


class SDMSAPIClient(
    BaseOSDUAPIClient,
    SDMSDatasetAPI,
    SDMSubprojectAPI,
    SDMSTenantAPI,
    SDMSUtilityAPI,
    SDMSAppsAPI,
    SDMSTokenAPI,
):
    service_path = "api/seismic-store/v3"



import os
from typing import AnyStr, Dict

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class SchemaAPIError(OSDUAPIError):
    pass


class SchemaAPIClient(BaseOSDUAPIClient):
    service_path = "api/schema-service/v1/schema"

    def get_schema(self, *, id: AnyStr, ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            f"{self.service_path}/{id}",
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise SchemaAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def get_schemas(self) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise SchemaAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def create_schema(
        self, *, schema: Dict
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
        )
        response = requests.post(
            url=url, headers=self.osdu_auth_backend.headers, json=schema
        )

        if not response.ok:
            raise SchemaAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def update_schema(
        self, *, schema: Dict
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
        )
        response = requests.put(
            url=url, headers=self.osdu_auth_backend.headers, json=schema
        )

        if not response.ok:
            raise SchemaAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()



from abc import ABCMeta, abstractmethod
from typing import AnyStr, Dict


class AuthBackendInterface(metaclass=ABCMeta):

    @property
    def headers(self) -> Dict:
        return self.get_headers()

    @property
    def base_url(self) -> AnyStr:
        return self.get_base_url()

    @abstractmethod
    def get_headers(self) -> Dict:
        pass

    @abstractmethod
    def get_base_url(self) -> AnyStr:
        pass

    @abstractmethod
    def get_sd_connection_params(self, log_level: int = None) -> Dict:
        pass



import os
from typing import AnyStr, Dict

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class LegalAPIException(OSDUAPIError):
    pass


class LegalAPIClient(BaseOSDUAPIClient):
    service_path = "api/legal/v1"

    def get_legal_tags(
        self,
        *,
        valid: bool = True,
        osdu_account_id: AnyStr = None,
        osdu_on_behalf_of: AnyStr = None,
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "legaltags",
        )
        params = {"valid": valid}
        headers = self.osdu_auth_backend.headers
        if osdu_account_id:
            headers["OSDU-Account-Id"] = osdu_account_id
        if osdu_on_behalf_of:
            headers["OSDU-On-Behalf-Of"] = osdu_on_behalf_of
        response = requests.get(url=url, headers=headers, params=params)

        if not response.ok:
            raise LegalAPIException(
                status_code=response.status_code, message=response.text
            )

        return response.json()



from .auth import AuthBackendInterface
from .exceptions import OSDUClientError
from .services.dataset_api import DatasetAPIClient
from .services.entitlements_api import EntitlementsAPIClient
from .services.legal_api import LegalAPIClient
from .services.schema_api import SchemaAPIClient
from .services.sdms_api import SDMSAPIClient
from .services.search_api import SearchAPIClient
from .services.storage_api import StorageAPIClient

CLIENTS = {
    "dataset": DatasetAPIClient,
    "entitlements": EntitlementsAPIClient,
    "schema": SchemaAPIClient,
    "sdms": SDMSAPIClient,
    "search": SearchAPIClient,
    "storage": StorageAPIClient,
    "legal": LegalAPIClient,
}


def get_service_client(name):
    if name not in CLIENTS:
        raise OSDUClientError(
            f"Service {name} not recognized. Choose one from available {', '.join(CLIENTS.keys())}"
        )
    return CLIENTS[name]


class OSDUAPI:
    @staticmethod
    def client(service_name, auth_backend: AuthBackendInterface = None):
        return get_service_client(service_name)(auth_backend)



from typing import AnyStr, Dict, Union

from ..exceptions import OSDUClientError


class OSDUAPIError(OSDUClientError):
    def __init__(self, status_code: int, message: Union[Dict, AnyStr] = None) -> None:
        self.status_code = status_code
        self.message = message
        super().__init__(self.message, self.status_code)



import os
from typing import AnyStr, Dict, List

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class DatasetAPIError(OSDUAPIError):
    pass


class DatasetAPIClient(BaseOSDUAPIClient):
    service_path = "api/dataset/v1"

    def get_storage_instructions(
        self, *, kind_sub_type: AnyStr
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "getStorageInstructions",
        )
        params = {"kindSubType": kind_sub_type}
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise DatasetAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def get_retrieval_instructions(
        self,
        *,
        dataset_registry_ids: List[AnyStr]
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "getRetrievalInstructions",
        )
        response = requests.post(
            url=url,
            headers=self.osdu_auth_backend.headers,
            json={"datasetRegistryIds": dataset_registry_ids},
        )

        if not response.ok:
            raise DatasetAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()



import os
from calendar import timegm
from datetime import datetime

import jwt
import requests

from osdu_client.auth import AuthBackendInterface


class AuthorizationError(Exception):
    pass


class Auth(AuthBackendInterface):
    def __init__(
        self,
        client_id,
        client_secret,
        auth_authorize_url,
        auth_token_url,
        iss,
        osduonaws_base_url,
        principal_client_id=None,
        principal_client_secret=None,
    ) -> None:
        self.iss = iss
        self.client_id = client_id
        self.client_secret = client_secret
        self.principal_client_id = principal_client_id
        self.principal_client_secret = principal_client_secret
        self.auth_authorize_url = auth_authorize_url
        self.auth_token_url = auth_token_url
        self.osduonaws_base_url = osduonaws_base_url

    def get_headers(self):
        return {
            "Authorization": f"Bearer {self.access_token}",
            "data-partition-id": self.data_partition_id,
        }

    def get_base_url(self):
        return self.osduonaws_base_url

    def get_service_principal_access_token(self) -> str:
        response = requests.post(
            url=self.auth_token_url,
            data={
                "grant_type": "client_credentials",
                "scope": "osduOnAws/osduOnAWSService",
            },
            auth=(self.principal_client_id, self.principal_client_secret),
        )
        if not response.ok:
            raise AuthorizationError(response.text)

        return response.json()

    def verify(self, access_token):
        response = requests.post(
            self.auth_token_url,
            auth=(self.client_id, self.client_secret),
            data={
                "grant_type": "urn:pingidentity.com:oauth2:grant_type:validate_bearer",
                "token": access_token,
                "client_id": self.client_id,
            },
        )
        if not response.ok:
            raise AuthorizationError("Token is invalid")

    def is_expired(self, raise_excpetion=True):
        if self.remaining_time <= 0:
            if raise_excpetion:
                raise AuthorizationError("User authorization token expired.")
            else:
                return True
        return False

    @property
    def remaining_time(self):
        return self.claims["exp"] - timegm(datetime.utcnow().utctimetuple())

    @staticmethod
    def decode(access_token):
        decoded = jwt.decode(access_token, options={"verify_signature": False})
        return decoded

    def is_authorized(self, verify=False, raise_exception=False):
        result = not self.is_expired(raise_excpetion=raise_exception)
        try:
            if verify:
                self.verify(self._access_token)
        except jwt.exceptions.PyJWTError as e:
            raise AuthorizationError(str(e)) from e

        return result

    def get_sd_connection_string(self, log_level=None):
        seismic_ddms_api = os.path.join(self.osduonaws_base_url, "api/seismic-store/v3")
        sd_conn_str = (
            f"sd_authority_url={seismic_ddms_api};"
            f"sd_api_key=xxx;auth_token_url={self.auth_token_url};"
            f"sdtoken={self.access_token};"
            f"client_id={self.client_id};client_secret={self.client_secret};"
            f"refresh_token={None};scopes=openid email"
        )
        if log_level:
            sd_conn_str += f";LogLevel={log_level}"

        return sd_conn_str



import os
from typing import AnyStr, Dict

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class EntitlementsAPIException(OSDUAPIError):
    pass


class EntitlementsAPIClient(BaseOSDUAPIClient):
    service_path = "api/entitlements/v2"

    def get_groups(self) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "groups",
        )
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise EntitlementsAPIException(
                status_code=response.status_code, message=response.text
            )

        return response.json()

    def get_members_groups(self, *, member_email, type="None") -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"members/{member_email}/groups",
        )
        params = {"type": type}
        response = requests.get(
            url=url, headers=self.osdu_auth_backend.headers, params=params
        )

        if not response.ok:
            raise EntitlementsAPIException(
                status_code=response.status_code, message=response.text
            )

        return response.json()

    def add_member(self, *, email: AnyStr, group_name: AnyStr, role: AnyStr) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            f"groups/{group_name}/members",
        )
        response = requests.post(
            url=url,
            headers=self.osdu_auth_backend.headers,
            json={"role": role, "email": email},
        )

        if not response.ok:
            raise EntitlementsAPIException(
                status_code=response.status_code, message=response.text
            )

        return response.json()

    def create_group(
        self,
        *,
        name: AnyStr,
        description: AnyStr,
        email: AnyStr,
        data_partition_id: AnyStr = None,
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            self.service_path,
            "groups",
        )
        data = {"name": name, "description": description, "email": email}
        headers = self.osdu_auth_backend.headers
        if data_partition_id:
            headers["data-partition-id"] = data_partition_id
        response = requests.post(url=url, headers=headers, json=data)

        if not response.ok:
            raise EntitlementsAPIException(
                status_code=response.status_code, message=response.text
            )

        return response.json()



from abc import ABCMeta

from ..auth import AuthBackendInterface


class BaseOSDUAPIClient(metaclass=ABCMeta):
    def __init__(self, osdu_auth_backend: AuthBackendInterface):
        self.osdu_auth_backend = osdu_auth_backend



import os
from typing import AnyStr, Dict, List

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class StorageAPIError(OSDUAPIError):
    pass


class StorageAPIClient(BaseOSDUAPIClient):

    def create_or_update_records(
        self, records: List[Dict]
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            "api/storage/v2/records/",
        )
        response = requests.put(
            url=url, headers=self.osdu_auth_backend.headers, json=records
        )

        if not response.ok:
            raise StorageAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def get_record(
        self,
        *,
        id: AnyStr,


    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            f"api/storage/v2/records/{id}",
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise StorageAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def delete_record(
        self,
        *,
        id: AnyStr
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            f"api/storage/v2/records/{id}",
        )
        response = requests.delete(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise StorageAPIError(
                status_code=response.status_code,
                message=response.text
            )

    def get_record_versions(
        self,
        *,
        id: AnyStr,
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            f"api/storage/v2/records/versions/{id}",
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise StorageAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def get_specific_record(
        self,
        *,
        versioned_id: AnyStr
    ) -> Dict:
        _id, version = versioned_id.rsplit(":", 1)
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            f"api/storage/v2/records/{_id}/{version}",
        )
        response = requests.get(url=url, headers=self.osdu_auth_backend.headers)

        if not response.ok:
            raise StorageAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def query_records(
        self,
        *,
        records: List[AnyStr]
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            "api/storage/v2/query/records",
        )
        response = requests.post(
            url=url,
            headers=self.osdu_auth_backend.headers,
            json={"records": records},
        )

        if not response.ok:
            raise StorageAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()



import os
from typing import AnyStr, Dict, List

import requests

from .base_api import BaseOSDUAPIClient
from .exceptions import OSDUAPIError


class SearchAPIError(OSDUAPIError):
    pass


class SearchAPIClient(BaseOSDUAPIClient):

    def search_query(
        self,
        *,
        kind: AnyStr,
        query: AnyStr,
        spatial_filter: Dict = None,
        returned_fields: List[AnyStr] = None,
        limit: int = 20,
        offset: int = 0,
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            "api/search/v2/query",
        )
        data = {
            "kind": kind,
            "query": query,
            "limit": limit,
            "offset": offset,
        }

        if spatial_filter:
            data["spatialFilter"] = spatial_filter

        if returned_fields:
            data["returnedFields"] = returned_fields

        response = requests.post(
            url=url, headers=self.osdu_auth_backend.headers, json=data
        )

        if not response.ok:
            raise SearchAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()

    def search_query_with_cursor(
        self,
        *,
        kind: AnyStr,
        query: AnyStr,
        spatial_filter: Dict = None,
        returned_fields: List[AnyStr] = None,
        cursor: AnyStr = None,
        limit: int = 20,
        offset: int = 0,
    ) -> Dict:
        url = os.path.join(
            self.osdu_auth_backend.base_url,
            "api/search/v2/query_with_cursor",
        )
        data = {
            "kind": kind,
            "query": query,
            "limit": limit,
            "offset": offset,
        }

        if spatial_filter:
            data["spatialFilter"] = spatial_filter

        if returned_fields:
            data["returnedFields"] = returned_fields

        if cursor:
            data["cursor"] = cursor

        response = requests.post(
            url=url, headers=self.osdu_auth_backend.headers, json=data
        )

        if not response.ok:
            raise SearchAPIError(
                status_code=response.status_code,
                message=response.text
            )

        return response.json()
